@@ -1,8 +1,8 @@
-/* A Bison parser, made by GNU Bison 3.7.6.  */
+/* A Bison parser, made by GNU Bison 3.5.1.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2021 Free Software Foundation,
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2020 Free Software Foundation,
    Inc.
 
    This program is free software: you can redistribute it and/or modify
@@ -16,7 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -31,9 +31,8 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-/* DO NOT RELY ON FEATURES THAT ARE NOT DOCUMENTED in the manual,
-   especially those whose name start with YY_ or yy_.  They are
-   private implementation details that can be changed or removed.  */
+/* Undocumented macros, especially those whose name start with YY_,
+   are private implementation details.  Do not rely on them.  */
 
 #ifndef YY_YY_WINPREFSYACC_H_INCLUDED
 # define YY_YY_WINPREFSYACC_H_INCLUDED
@@ -45,52 +44,43 @@
 extern int yydebug;
 #endif
 
-/* Token kinds.  */
+/* Token type.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
   enum yytokentype
   {
-    YYEMPTY = -2,
-    YYEOF = 0,                     /* "end of file"  */
-    YYerror = 256,                 /* error  */
-    YYUNDEF = 257,                 /* "invalid token"  */
-    NEWLINE = 258,                 /* NEWLINE  */
-    MENU = 259,                    /* MENU  */
-    LB = 260,                      /* LB  */
-    RB = 261,                      /* RB  */
-    ICONDIRECTORY = 262,           /* ICONDIRECTORY  */
-    DEFAULTICON = 263,             /* DEFAULTICON  */
-    ICONS = 264,                   /* ICONS  */
-    STYLES = 265,                  /* STYLES  */
-    TOPMOST = 266,                 /* TOPMOST  */
-    MAXIMIZE = 267,                /* MAXIMIZE  */
-    MINIMIZE = 268,                /* MINIMIZE  */
-    BOTTOM = 269,                  /* BOTTOM  */
-    NOTITLE = 270,                 /* NOTITLE  */
-    OUTLINE = 271,                 /* OUTLINE  */
-    NOFRAME = 272,                 /* NOFRAME  */
-    DEFAULTSYSMENU = 273,          /* DEFAULTSYSMENU  */
-    SYSMENU = 274,                 /* SYSMENU  */
-    ROOTMENU = 275,                /* ROOTMENU  */
-    SEPARATOR = 276,               /* SEPARATOR  */
-    ATSTART = 277,                 /* ATSTART  */
-    ATEND = 278,                   /* ATEND  */
-    EXEC = 279,                    /* EXEC  */
-    ALWAYSONTOP = 280,             /* ALWAYSONTOP  */
-    DEBUGOUTPUT = 281,             /* "DEBUG"  */
-    RELOAD = 282,                  /* RELOAD  */
-    TRAYICON = 283,                /* TRAYICON  */
-    FORCEEXIT = 284,               /* FORCEEXIT  */
-    SILENTEXIT = 285,              /* SILENTEXIT  */
-    STRING = 286                   /* STRING  */
+    NEWLINE = 258,
+    MENU = 259,
+    LB = 260,
+    RB = 261,
+    ICONDIRECTORY = 262,
+    DEFAULTICON = 263,
+    ICONS = 264,
+    STYLES = 265,
+    TOPMOST = 266,
+    MAXIMIZE = 267,
+    MINIMIZE = 268,
+    BOTTOM = 269,
+    NOTITLE = 270,
+    OUTLINE = 271,
+    NOFRAME = 272,
+    DEFAULTSYSMENU = 273,
+    SYSMENU = 274,
+    ROOTMENU = 275,
+    SEPARATOR = 276,
+    ATSTART = 277,
+    ATEND = 278,
+    EXEC = 279,
+    ALWAYSONTOP = 280,
+    DEBUGOUTPUT = 281,
+    RELOAD = 282,
+    TRAYICON = 283,
+    FORCEEXIT = 284,
+    SILENTEXIT = 285,
+    STRING = 286
   };
-  typedef enum yytokentype yytoken_kind_t;
 #endif
-/* Token kinds.  */
-#define YYEMPTY -2
-#define YYEOF 0
-#define YYerror 256
-#define YYUNDEF 257
+/* Tokens.  */
 #define NEWLINE 258
 #define MENU 259
 #define LB 260
@@ -131,7 +121,7 @@ union YYSTYPE
   unsigned long uVal;
   int iVal;
 
-#line 135 "winprefsyacc.h"
+#line 125 "winprefsyacc.h"
 
 };
 typedef union YYSTYPE YYSTYPE;
