@@ -51,7 +51,7 @@
 void
 shadowUpdateRotatePacked(ScreenPtr pScreen, shadowBufPtr pBuf)
 {
-    RegionPtr damage = shadowDamage(pBuf);
+    RegionPtr damage = DamageRegion(pBuf->pDamage);
     PixmapPtr pShadow = pBuf->pPixmap;
     int nbox = RegionNumRects(damage);
     BoxPtr pbox = RegionRects(damage);
@@ -259,7 +259,7 @@ shadowUpdateRotatePacked(ScreenPtr pScreen, shadowBufPtr pBuf)
                     p = pixelsPerBits;
                     /*
                      * Build one word of output from multiple inputs
-                     * 
+                     *
                      * Note that for 90/270 rotations, this will walk
                      * down the shadow hitting each scanline once.
                      * This is probably not very efficient.
@@ -296,9 +296,3 @@ shadowUpdateRotatePacked(ScreenPtr pScreen, shadowBufPtr pBuf)
         }
     }
 }
-
-shadowUpdateProc
-shadowUpdateRotatePackedWeak(void)
-{
-    return shadowUpdateRotatePacked;
-}
